from edge.support import fs
from edge.types.exceptions import GenericError
from flask import Flask, request, jsonify
from flask_jwt_extended import JWTManager, create_access_token, jwt_required
from werkzeug.security import check_password_hash

app = Flask(__name__)


def _load_secret() -> str:
    with open('/etc/edge/uzsc.0/secret', 'r') as text_file:
        secret = text_file.readline()

    return secret.strip()


def _load_users() -> dict[str, str]:
    jnode = fs.load_json3('/etc/edge/uzsc.0/users.conf', default=None)
    if jnode is None:
        return {}

    return {
        user['user']: user['pass'] for user in jnode['users']
    }


# Setup the Flask-JWT-Extended extension
app.config['JWT_SECRET_KEY'] = _load_secret()
app.config['JWT_ERROR_MESSAGE_KEY'] = 'message'
jwt = JWTManager(app)


@app.route('/auth', methods=['POST'])
def authenticate():
    """Authentication endpoint to generate JWT token."""
    data = request.get_json()

    user_name = data.get('username')
    user_pass = data.get('password')

    if not user_name or not user_pass:
        return jsonify({
            'code': 400,
            'type': 'ValidationError',
            'message': 'Username and password are required'
        }), 400

    try:
        user_to_hash_map: dict[str, str] = _load_users()
        if user_name not in user_to_hash_map or not check_password_hash(user_to_hash_map[user_name], user_pass):
            return jsonify({
                'code': 401,
                'type': 'AuthenticationError',
                'message': 'Invalid credentials'
            }), 401
    except GenericError:
        return jsonify({
            'code': 500,
            'type': 'ServerError',
            'message': 'Internal server error',
            'error': ''
        }), 500
    except Exception as x:
        return jsonify({
            'code': 500,
            'type': 'ServerError',
            'message': 'Internal server error',
            'error': str(x)
        }), 500

    # Generate JWT token
    token = create_access_token(identity=user_name)

    return jsonify({
        'code': 200,
        'message': 'Successfully logged in!',
        'token': token
    }), 200

@app.route('/controller/switch_by_code', methods=['POST'])
@jwt_required()
def switch_controller():
    """Controller endpoint to change the event policy."""
    data = request.get_json()

    status = data.get('status')
    if status not in ['on', 'off']:
        return jsonify({
            'code': 400,
            'type': 'ValidationError',
            'message': 'Invalid body#/status'
        }), 400

    try:
        from edge.webserver.sysconf.api_controller import ControllerApi
        event_policy = 'all' if status == 'on' else 'none'
        ControllerApi.set_policy(event_policy)

        return jsonify({
            'code': 200,
            'done': True
        }), 200

    except Exception as x:
        return jsonify({
            'code': 500,
            'type': 'ServerError',
            'message': 'Internal server error',
            'error': str(x)
        }), 500


if __name__ == '__main__':
    app.logger.info('exec: run')
    app.run(debug=False)
    app.logger.info('done: run')
